"""
This module contains the HTTPSERVER(for python) for
uploading CSV only files and also uses csv_db.py module
(for creating generic db table)
and crud_operation_db module(for applying CRUD operations)
"""
import os

from http.server \
    import HTTPServer, \
    BaseHTTPRequestHandler
import cgi
import logging
from mysql import connector
import jinja2

logging.basicConfig(filename='CRUD_operation.log',
                    level=logging.INFO,
                    format='%(asctime)s: %(levelname)s:'
                           ' %(filename)s->'
                           ' %(funcName)s->'
                           ' Line %(lineno)d-> %(message)s')

from csv_db import csv_to_db as file_db
from crud_operation_db \
    import view_db_data, \
    delete_db_data, \
    update_db_data, \
    create_db_data

default_file = 'Truth_folder/uploadFile_csv.csv'


class HttpRequestToResponse(BaseHTTPRequestHandler):
    """
    This class containing methods such as do_GET() do_POST()
    that are specifically useful for local server testing.

     Attributes
     ------------------------------
     FILE = name of the file user wats to use to save the data into.
            default set to uploadFile_csv.csv
    filter_data = A dict parameter used in do_POST() method for
                 filtering the table data when required.
    """
    global default_file
    filter_data = {}
    return_msg = None

    def do_GET(self):
        """The function to interact with db and file
        uploading functionality on client side browser."""
        try:
            if self.path.endswith('/'):
                self.send_response(200, message="this is message")
                self.send_header('content-type', 'text/html')
                self.end_headers()
                with open('Templates/index.html', 'r', encoding="utf-8") as index_html:
                    output = index_html.read()
                self.wfile.write(output.encode())
            if self.path.endswith('/upload'):
                self.send_response(200)
                self.send_header('content-type', 'text/html')
                self.end_headers()

                with open('Templates/uploadfile.html', 'r', encoding="utf-8") as upload_file:
                    output = upload_file.read()
                self.wfile.write(output.encode())

            if self.path.endswith('/view'):
                self.send_response(200)
                self.send_header('content-type', 'text/html')
                self.end_headers()
                table_name = os.path.splitext(default_file)[0].lower()
                table = view_db_data('fileupload', table_name,
                                     filter_data=self.filter_data)

                if table is not None:
                    with open('Templates/update_and_delete_row.html', 'r', encoding='utf-8')\
                            as view_file:
                        output = view_file.read()
                    render_output = jinja2.Template(output)
                    self.wfile.write((render_output.render(table=table).encode()))

            if self.path.endswith('/add'):
                self.send_response(200)
                self.send_header('content-type', 'text/html')
                self.end_headers()
                table_name = os.path.splitext(default_file)[0].lower()
                conn = connector.connect(host='localhost',
                                         user='root',
                                         password='saatwik')

                if conn.is_connected():
                    cursor = conn.cursor()
                    cursor.execute(f'SELECT * FROM fileupload.{table_name}')
                    db_column = cursor.column_names
                    with open('Templates/create_new_row.html', 'r', encoding='utf-8')\
                            as create_row_file:
                        output = create_row_file.read()
                    render_output = jinja2.Template(output)
                self.wfile.write(render_output.render(db_column=db_column).encode())

        except connector.ProgrammingError as prog_err:
            logging.error("%s: %s", prog_err.__class__.__name__, prog_err)
            self.send_error(code=prog_err.errno, message=f'{prog_err}')
        except connector.Error as conn_err:
            logging.error("%s: %s", conn_err.__class__.__name__, conn_err)

    def do_POST(self):
        """The function to upload the data in the db and
        perform crud operation using client browser."""
        global default_file
        try:
            if self.path.endswith('/upload'):
                c_type, pdict = cgi.parse_header(self.headers.get('content-type'))
                pdict['boundary'] = bytes(pdict['boundary'], "utf-8")
                content_len = int(self.headers.get('Content-length'))
                pdict['CONTENT-LENGTH'] = content_len
                if c_type == 'multipart/form-data':
                    fields = cgi.parse_multipart(self.rfile, pdict)
                    default_file = fields.get('your_filename')[0] \
                        if fields.get('your_filename')[0] != '' else default_file
                    filedata = fields.get('filedata')[0]
                    filedata = filedata.decode("utf-8-sig")
                    with open(default_file, mode="w", encoding="utf-8-sig") as newfile:
                        for data in filedata.split('\r\r'):
                            newfile.write(f"{data}")
                    if 'pk_field_datatype' in fields and 'primary_key' in fields:
                        file_db(filename=default_file,
                                pk_field_dtype=fields.get('pk_field_datatype')[0],
                                primary_key_field=fields.get('primary_key')[0])
                    else:
                        file_db(filename=default_file)
                self.send_response(301)
                self.send_header('content-type', 'text-html')
                self.send_header('Location', '/upload')
                self.end_headers()

            if self.path.endswith('/view'):
                table_name = os.path.splitext(default_file)[0].lower()
                c_type, pdict = cgi.parse_header(self.headers.get('content-type'))
                pdict['boundary'] = bytes(pdict['boundary'], "utf-8")
                content_len = int(self.headers.get('Content-length'))
                pdict['CONTENT-LENGTH'] = content_len
                if c_type == 'multipart/form-data':
                    fields = cgi.parse_multipart(self.rfile, pdict)
                    if 'textfield' in fields.keys():
                        columns = fields.get('textfield')[0]
                        self.filter_data['columns'] = columns
                    if 'expression_field' in fields.keys():
                        expression = fields.get('expression_field')[0]
                        expression = expression.split('=')
                        self.return_msg = delete_db_data('fileupload', table_name,
                                                         expression=expression)
                        # if isinstance(self.RETURN_MSG, tuple):
                        #     self.send_error(self.RETURN_MSG[0], message=self.RETURN_MSG[1])
                    if 'set_field' in fields.keys() and 'target_field' in fields.keys():
                        set_value = fields.get('set_field')[0]
                        set_value = set_value.split('=')
                        target_value = fields.get('target_field')[0]
                        target_value = target_value.split('=')
                        update_db_data('fileupload', table_name,
                                       set_value, target_value)
                self.send_response(301)
                self.send_header('content-type', 'text-html')
                self.send_header('Location', '/view')
                self.end_headers()

            if self.path.endswith('/add'):
                table_name = os.path.splitext(default_file)[0].lower()
                c_type, pdict = cgi.parse_header(self.headers.get('content-type'))
                pdict['boundary'] = bytes(pdict['boundary'], "utf-8")
                content_len = int(self.headers.get('Content-length'))
                pdict['CONTENT-LENGTH'] = content_len
                if c_type == 'multipart/form-data':
                    fields = cgi.parse_multipart(self.rfile, pdict)
                    fields = {key: fields[key][0] for key in fields}
                    row_values = fields
                    create_db_data('fileupload', table_name, row_values=row_values)

                self.send_response(301)
                self.send_header('content-type', 'text-html')
                self.send_header('Location', '/view')
                self.end_headers()
        except IndexError as ind_err:
            logging.error('%s: %s', ind_err.__class__.__name__, ind_err)
            self.send_error(code=406, message=f'{ind_err}')
        except PermissionError as perm_err:
            logging.error('%s:%s', perm_err.__class__.__name__, perm_err)
        except TypeError as type_err:
            logging.error('%s: %s',
                          type_err.__class__.__name__, type_err)
        except connector.ProgrammingError as prog_err:
            logging.error('%s: %s',
                          prog_err.__class__.__name__, prog_err)
            self.send_response(301)
            self.send_header('content-type', 'text-html')
            self.send_header('Location', '/view')
            self.end_headers()


def main():
    """Function to start the HTTP server using python script"""
    port = 8000
    # HttpRequestToResponse.FILE = 'museum_CSV.csv'
    server = HTTPServer(('localhost', port), HttpRequestToResponse)
    print("Server started on localhost: ", port)
    server.serve_forever()


if __name__ == "__main__":
    main()



columns = ['id', 'title', 'original_title', 'original_title_romanised',
           'image', 'movie_banner', 'description', 'director',
           'producer', 'release_date', 'running_time', 'rt_score',
           'people', 'species', 'locations', 'vehicles', 'url'
           ]

# ----------------------------------------FOR INSERTION-----------------------------------

Value1 = {columns[0]: 'id_3', columns[1]: 'iuwrruy', columns[2]: 'iauf arhyrgfds',
          columns[3]: 'rhwouervh', columns[4]: 'some.jpg', columns[5]: 'other.jpg',
          columns[6]: 'a movie', columns[7]: 'argrghu', columns[8]: 'isrrig',
          columns[9]: 2021, columns[10]: 29, columns[11]: 80,
          columns[12]: 'a,list,of,people', columns[13]: 'Human', columns[14]: 'paris',
          columns[15]: 'car', columns[16]: 'uiwarrffurq'}

Value2 = {columns[0]: 'id_4', columns[1]: 'iuwzdffbgrruy', columns[2]: 'idfdggfds',
          columns[3]: 'rhsfgarrvh', columns[4]: 'sosfgde.jpg', columns[5]: 'ofgdaher.jpg',
          columns[6]: 'asdfdgvie', columns[7]: 'arsdgdghu', columns[8]: 'isadfdig',
          columns[9]: 2056, columns[10]: 79, columns[11]: 455,
          columns[12]: 'a,list,of,people', columns[13]: 'Human,cat', columns[14]: 'paris',
          columns[15]: 'car', columns[16]: 'uiwarrffurq'}

# Same id
Value3 = {columns[0]: 'id_2', columns[1]: 'vroom', columns[2]: 'vroom_romanised',
          columns[3]: 'rhsfgarrvh', columns[4]: 'sosfgde.jpg', columns[5]: 'ofgdaher.jpg',
          columns[6]: 'asdfdgvie', columns[7]: 'arsdgdghu', columns[8]: 'isadfdig',
          columns[9]: 2056, columns[10]: 79, columns[11]: 455,
          columns[12]: 'a,list,of,people', columns[13]: 'Human,cat', columns[14]: 'paris',
          columns[15]: 'car', columns[16]: 'uiwarrffurq'}

Value5 = {columns[0]: 'id_6', columns[1]: 'vroom', columns[2]: 'vroom_romanised',
          columns[3]: 'rhsfgarrvh', columns[4]: 'sosfgde.jpg', columns[5]: 'ofgdaher.jpg',
          columns[6]: 'asdfdgvie', columns[7]: 'arsdgdghu', columns[8]: 'isadfdig',
          columns[9]: '2056', columns[10]: 79, columns[11]: 455,
          columns[12]: 'a,list,of,people', columns[13]: 'Human,cat', columns[14]: 'paris',
          columns[15]: 'car', columns[16]: 'uiwarrffurq'}

Value4 = {columns[0]: 'id_5', columns[1]: 'vroom', columns[2]: 'vroom_romanised',
          columns[3]: 'rhsfgarrvh', columns[4]: 'sosfgde.jpg', columns[5]: 'ofgdaher.jpg',
          columns[6]: 'asdfdgvie', columns[7]: 'arsdgdghu', columns[8]: 'isadfdig'}

# columns[9]: 2056, columns[10]: 79, columns[11]: 455,
# columns[12]: 'a,list,of,people', columns[13]: 'Human,cat', columns[14]: 'paris',
# columns[15]: 'car', columns[16]: 'uiwarrffurq'}

# Extra column
Value6 = {columns[0]: 'id_7', columns[1]: 'vroom', columns[2]: 'vroom_romanised',
          columns[3]: 'rhsfgarrvh', columns[4]: 'sosfgde.jpg', columns[5]: 'ofgdaher.jpg',
          columns[6]: 'asdfdgvie', columns[7]: 'arsdgdghu', columns[8]: 'isadfdig',
          columns[9]: '2056', columns[10]: 79, columns[11]: 455,
          columns[12]: 'a,list,of,people', columns[13]: 'Human,cat', columns[14]: 'paris',
          columns[15]: 'car', columns[16]: 'uiwarrffurq', "extra": 456534}

# ------------------------------------------END--------------------------------------------



""" This module contains the unit testing of CRUD operation functions.
 It includes both negative test cases and positive test case"""
import mysql.connector

from src.csv_to_db_converter.crud_operation_db import view_db_data, create_db_data, update_db_data, delete_db_data
from src.csv_to_db_converter.tests.FakeDB import My_DB
from src.csv_to_db_converter.tests.myfakedata import Value1, Value2, Value3, Value4, Value6

EMPTY_LIST = []

WRONG_DB_NAME = 1234

MYSQL_DB = 'Fake_DB'

DB_TABLE = 'test_table'


class TestCRUD(My_DB):
    """ This class contains test cases for CRUD operation functions,
    it includes how each function will be handling the inputs given by user.
    Possible Test Cases (positive and negative) are implemented here.
    """

    def test_access_db_unsuccessful(self):
        """ Check if the Database name is in correct and table name is in correct """
        with self.mock_db_config:
            self.assertEqual(create_db_data(db_name=WRONG_DB_NAME,
                                            db_table='None', row_values=Value1), "ProgrammingError")
            self.assertEqual(create_db_data(db_name='None', db_table='None',
                                            row_values=Value1), "ProgrammingError")

    def test_insert_in_db_successful(self):
        """ Check if Inserting data in database table is successful """
        with self.mock_db_config:
            self.assertEqual(create_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, row_values=Value1),
                             "Successfully inserted")

            self.assertEqual(create_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, row_values=Value2),
                             "Successfully inserted")

            self.assertEqual(create_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, row_values=Value4),
                             "Successfully inserted")

    def test_insert_in_db_unsuccessful(self):
        """ Check if Inserting data in database table is unsuccessful """
        with self.mock_db_config:
            self.assertEqual(create_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, row_values=Value3),
                             "Integrity error")
            self.assertEqual(create_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, row_values=Value6),
                             "ProgrammingError")
            self.assertEqual(create_db_data(db_name=MYSQL_DB
                                            , db_table=DB_TABLE, row_values=EMPTY_LIST),
                             "AttributeError")

    def test_delete_row_in_db_successful(self):
        """ Check if deleting rows in database is successful"""
        with self.mock_db_config:
            self.assertEqual(delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                            expression=['id', 'id_1']), "Deleted successfully")

            self.assertEqual(delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                            expression=['rt_score', '80']), "Deleted successfully")

            self.assertEqual(delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                            expression=['vehicles', 'car']), "Deleted successfully")

    def test_delete_row_in_db_unsuccessful(self):
        """ Check if deleting rows in database is successful """

        self.assertEqual(delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                        expression=['ids', 'id_5']), "Programming error")
        self.assertEqual(delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                        expression=['id', '15']), "DataError error")
        with self.assertRaises(TypeError):
            delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, expression=123)
        with self.assertRaises(TypeError):
            delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, expression='')
        with self.assertRaises(TypeError):
            delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE, expression=[3356])
        with self.assertRaises(TypeError):
            delete_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                           expression=[{'setgwett': 'asddgf vqeet'}])

    def test_read_from_db_successfully_finds_rows(self):
        """ Checking if Read operation for
         the Database is successful - the data being looked is successfully returned """
        with self.mock_db_config:
            self.assertIsNotNone(view_db_data(db_name=MYSQL_DB, db_table=DB_TABLE))

            self.assertIsNotNone(view_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                              filter_data={'columns': 'id,title,'
                                                                      'original_title'}))
            self.assertIsNotNone(view_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                              filter_data={'columns': 'id,title,'
                                                                      'original_title',
                                                           'where': 'id="id_2"'}))

    def test_read_from_db_unsuccessful(self):
        """ Checking if Read operation for the Database is unsuccessful """
        with self.assertRaises(mysql.connector.ProgrammingError):
            self.assertIsNotNone(view_db_data(db_name='Wrong database', db_table='wrong tanle'))

        with self.assertRaises(mysql.connector.ProgrammingError):
            self.assertIsNotNone(view_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                              filter_data={'columns': 'id,tititit,original_title'}))

        with self.assertRaises(mysql.connector.ProgrammingError):
            self.assertIsNotNone(view_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                              filter_data={'columns': 'id,title,original_title',
                                                           'where': 'id=e3e'}))

    def test_update_in_db_successful(self):
        """ Check if Update operation for the Database is successful -
         Data is updated successfully"""
        with self.mock_db_config:
            self.assertEqual(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                            set_value=["original_title", "an original title"],
                                            target_exp=['id', 'id_4']),
                             "Update successful")

            self.assertEqual(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                            set_value=["original_title", "an original title"],
                                            target_exp=['movie_banner', 'banner1.jpg']),
                             "Update successful")

    def test_update_in_db_unsuccessful(self):
        """ Checking if Update operation for the Database is unsuccessful -
                 data is not updated"""

        self.assertIsNone(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE))
        self.assertEqual(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                        set_value=["original_title", "an original title"]),
                         None)
        self.assertEqual(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                        set_value=[], target_exp=[]),
                         None)
        self.assertIsNone(update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                                         set_value=34567, target_exp=576547))

        with self.assertRaises(mysql.connector.IntegrityError):
            update_db_data(db_name=MYSQL_DB, db_table=DB_TABLE,
                           set_value=['id', 'id_2'],
                           target_exp=['title', 'iuwzdffbgrruy'])
